from __future__ import unicode_literals

import os

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
from model_utils import Choices
from model_utils.models import TimeStampedModel

from quiz.managers import QuizQuerySet


@python_2_unicode_compatible
class AbstractQuiz(TimeStampedModel):
    name = models.CharField(max_length=150)
    start_with_signup = models.BooleanField(default=True)

    objects = QuizQuerySet.as_manager()

    class Meta:
        abstract = True
        app_label = 'quiz'
        verbose_name = _("Quiz")
        verbose_name_plural = _("Quizzes")

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class AbstractQuestion(models.Model):
    OPEN, CHOICE = 'text', 'choice'
    QUESTION_TYPE_CHOICES = Choices(
        (CHOICE, _("Multiple choice")),
        (OPEN, _("Open question")),
    )

    quiz = models.ForeignKey('quiz.Quiz', related_name='questions')
    question = models.CharField(max_length=150)
    question_type = models.CharField(
        max_length=50, choices=QUESTION_TYPE_CHOICES, default=CHOICE)
    position = models.IntegerField(default=0)

    class Meta:
        abstract = True
        app_label = 'quiz'
        ordering = ['position']
        verbose_name = _("Question")
        verbose_name_plural = _("Questions")

    def __str__(self):
        return self.question


def handle_upload_answer_image(instance, filename):
    """Handle the upload filename / location for our answer image."""
    fname, ext = os.path.splitext(filename)
    return os.path.join(
        'quiz', slugify(instance.question.quiz.name),
        '%s%s' % (slugify(instance.answer), ext))


@python_2_unicode_compatible
class AbstractAnswerChoice(models.Model):
    question = models.ForeignKey(
        'quiz.Question', related_name='available_answers')
    answer = models.CharField(max_length=150)
    image = models.ImageField(
        blank=True, null=True, upload_to=handle_upload_answer_image)
    position = models.IntegerField(default=0)

    class Meta:
        abstract = True
        app_label = 'quiz'
        ordering = ['position']
        verbose_name = _("Choice")
        verbose_name_plural = _("Choices")

    def __str__(self):
        return self.answer


@python_2_unicode_compatible
class AbstractQuizResult(TimeStampedModel):
    quiz = models.ForeignKey('quiz.Quiz', related_name='results')
    name = models.CharField(max_length=150)
    email = models.EmailField()
    date_participated = models.DateField(auto_now_add=True)

    class Meta:
        abstract = True
        app_label = 'quiz'
        verbose_name = _("Quiz Result")
        verbose_name_plural = _("Quiz Results")

    def __str__(self):
        return self.name


class AbstractQuizResultItem(TimeStampedModel):
    quiz_result = models.ForeignKey(
        'quiz.QuizResult', related_name='result_items')
    question = models.ForeignKey('quiz.Question', related_name='+')
    answer_text = models.TextField(blank=True, null=True)
    answer_choice = models.ForeignKey(
        'quiz.AnswerChoice', related_name='+', blank=True, null=True)

    class Meta:
        abstract = True
        app_label = 'quiz'
        verbose_name = _("Quiz Result Item")
        verbose_name_plural = _("Quiz Result Items")



from django.apps import apps


def is_model_registered(app_label, model_name):
    """Checks whether a given model is registered.

    This is used to only register models if they aren't
    overridden by the application

    """
    try:
        apps.get_registered_model(app_label, model_name)
    except LookupError:
        return False
    else:
        return True



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-07 06:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import quiz.abstract_models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AnswerChoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('answer', models.CharField(max_length=150)),
                ('image', models.ImageField(blank=True, null=True, upload_to=quiz.abstract_models.handle_upload_answer_image)),
                ('position', models.IntegerField(default=0)),
            ],
            options={
                'ordering': ['position'],
                'abstract': False,
                'verbose_name': 'Choice',
                'verbose_name_plural': 'Choices',
            },
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('question', models.CharField(max_length=150)),
                ('question_type', models.CharField(choices=[('choice', 'Multiple choice'), ('text', 'Open question')], default='choice', max_length=50)),
                ('position', models.IntegerField(default=0)),
            ],
            options={
                'ordering': ['position'],
                'abstract': False,
                'verbose_name': 'Question',
                'verbose_name_plural': 'Questions',
            },
        ),
        migrations.CreateModel(
            name='Quiz',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=150)),
                ('start_with_signup', models.BooleanField(default=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Quiz',
                'verbose_name_plural': 'Quizzes',
            },
        ),
        migrations.CreateModel(
            name='QuizResult',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=150)),
                ('email', models.EmailField(max_length=254)),
                ('date_participated', models.DateField(auto_now_add=True)),
                ('quiz', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='results', to='quiz.Quiz')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Quiz Result',
                'verbose_name_plural': 'Quiz Results',
            },
        ),
        migrations.CreateModel(
            name='QuizResultItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('answer_text', models.TextField(blank=True, null=True)),
                ('answer_choice', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='quiz.AnswerChoice')),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='quiz.Question')),
                ('quiz_result', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='result_items', to='quiz.QuizResult')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Quiz Result Item',
                'verbose_name_plural': 'Quiz Result Items',
            },
        ),
        migrations.AddField(
            model_name='question',
            name='quiz',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='questions', to='quiz.Quiz'),
        ),
        migrations.AddField(
            model_name='answerchoice',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='available_answers', to='quiz.Question'),
        ),
    ]



from django.apps import apps
from django.contrib import admin

AnswerChoice = apps.get_model('quiz', 'AnswerChoice')
Question = apps.get_model('quiz', 'Question')
Quiz = apps.get_model('quiz', 'Quiz')
QuizResult = apps.get_model('quiz', 'QuizResult')
QuizResultItem = apps.get_model('quiz', 'QuizResultItem')


class QuizAdmin(admin.ModelAdmin):
    list_display = ['name']
admin.site.register(Quiz, QuizAdmin)


class AnswerChoiceAdminInline(admin.TabularInline):
    model = AnswerChoice
    extra = 0


class QuestionAdmin(admin.ModelAdmin):
    inlines = [AnswerChoiceAdminInline]
    list_display = ['question', 'question_type', 'position']
    list_editable = ['position']
    list_filter = ['quiz__name']
admin.site.register(Question, QuestionAdmin)


class QuizResultItemAdminInline(admin.TabularInline):
    model = QuizResultItem
    extra = 0


class QuizResultAdmin(admin.ModelAdmin):
    inlines = [QuizResultItemAdminInline]
    list_display = ['name', 'email', 'created']
    list_filter = ['quiz']
admin.site.register(QuizResult, QuizResultAdmin)



from django import forms
from django.apps import apps

Question = apps.get_model('quiz', 'Question')
Quiz = apps.get_model('quiz', 'Quiz')
QuizResult = apps.get_model('quiz', 'QuizResult')
QuizResultItem = apps.get_model('quiz', 'QuizResultItem')


class QuizResultItemForm(forms.ModelForm):
    """Form to save the result of each `Question` in a `QuizResultItem`"""

    class Meta:
        model = QuizResultItem
        fields = [
            'question',
            'answer_text',
            'answer_choice',
        ]
        widgets = {
            'question': forms.HiddenInput,
            'answer_choice': forms.RadioSelect,
        }

    def __init__(self, *args, **kwargs):
        self.question = kwargs['initial'].get('question')
        super(QuizResultItemForm, self).__init__(*args, **kwargs)

        if self.question.question_type == Question.CHOICE:
            self.fields['answer_choice'].empty_label = None
            self.fields['answer_choice'].queryset = (
                self.question.available_answers.all())
            del self.fields['answer_text']
        else:
            del self.fields['answer_choice']

        del self.fields['question']

    def get_question_text(self):
        return self.question.question

    def clean(self):
        data = super(QuizResultItemForm, self).clean()
        data['question'] = self.question
        return data

    def save(self, result, commit=True):
        result_item = super(QuizResultItemForm, self).save(commit=False)
        result_item.quiz_result = result
        result_item.save()
        return result_item


class QuizResultForm(forms.ModelForm):

    class Meta:
        model = QuizResult
        fields = [
            'quiz',
            'name',
            'email',
        ]
        widgets = {
            'quiz': forms.HiddenInput,
        }



from collections import OrderedDict

from django.apps import apps
from django.conf import settings
from django.http import HttpResponseRedirect, Http404
from django.urls import reverse
from django.views.generic import TemplateView
from formtools.wizard.views import SessionWizardView

from quiz import forms

Quiz = apps.get_model('quiz', 'Quiz')
Question = apps.get_model('quiz', 'Question')


class QuizWizard(SessionWizardView):
    template_name = 'quiz/wizard.html'
    item_form = forms.QuizResultItemForm
    signup_form = forms.QuizResultForm
    form_list = [
        forms.QuizResultForm,
    ]

    def dispatch(self, request, *args, **kwargs):
        self.object = Quiz.objects.get_current()
        if not self.object:
            raise Http404("Quiz doesn't exist.")
        return super(QuizWizard, self).dispatch(request, *args, **kwargs)

    def get_form_list(self):
        form_list = OrderedDict()
        if self.object.start_with_signup:
            form_list['0'] = self.signup_form

        for i, q in enumerate(self.object.questions.all(), len(form_list)):
            form_list[str(i)] = self.item_form

        if not self.object.start_with_signup:
            form_list[str(len(form_list))] = self.signup_form

        return form_list

    def get_form_initial(self, step):
        intstep = int(step)
        questions = list(self.object.questions.all())

        if (
            self.object.start_with_signup and intstep == 0 or
            not self.object.start_with_signup and intstep == len(questions)
        ):
            return {'quiz': self.object}

        return {
            'question': questions[intstep - 1]
        }

    def get_form(self, step=None, data=None, files=None):
        """
        This method was copied from the base Django 1.6 wizard
        class in order to support a callable `get_form_class`
        method which allows dynamic modelforms.
        """
        if step is None:
            step = self.steps.current

        form_list = self.get_form_list()
        kwargs = self.get_form_kwargs(step)
        kwargs.update({
            'data': data,
            'files': files,
            'initial': self.get_form_initial(step),
        })

        return form_list[step](**kwargs)

    def done(self, form_list, **kwargs):
        first = self.object.start_with_signup
        result_form = form_list.pop(0) if first else form_list.pop()
        result = result_form.save()

        for form in form_list:
            form.save(result)

        return HttpResponseRedirect(
            getattr(settings, 'QUIZ_REDIRECT_URL', reverse('quiz-thankyou')))


class ThankYouView(TemplateView):
    template_name = 'quiz/thankyou.html'

    def dispatch(self, request, *args, **kwargs):
        self.object = Quiz.objects.get_current()
        if not self.object:
            raise Http404("Quiz doesn't exist.")
        return super(ThankYouView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(ThankYouView, self).get_context_data(**kwargs)
        ctx['object'] = self.object
        return ctx



from quiz import abstract_models
from quiz.utils import is_model_registered

__all__ = []


if not is_model_registered('simple_quiz', 'Quiz'):
    class Quiz(abstract_models.AbstractQuiz):
        pass

    __all__.append('Quiz')


if not is_model_registered('simple_quiz', 'Question'):
    class Question(abstract_models.AbstractQuestion):
        pass

    __all__.append('Question')


if not is_model_registered('simple_quiz', 'AnswerChoice'):
    class AnswerChoice(abstract_models.AbstractAnswerChoice):
        pass

    __all__.append('AnswerChoice')


if not is_model_registered('simple_quiz', 'QuizResult'):
    class QuizResult(abstract_models.AbstractQuizResult):
        pass

    __all__.append('QuizResult')


if not is_model_registered('simple_quiz', 'QuizResultItem'):
    class QuizResultItem(abstract_models.AbstractQuizResultItem):
        pass

    __all__.append('QuizResultItem')



from django.conf.urls import url

from quiz import views

urlpatterns = [
    url(r'^$', views.QuizWizard.as_view(), name='quiz-wizard'),
    url(r'^thankyou/$', views.ThankYouView.as_view(), name='quiz-thankyou'),
]



import os
import sys

from setuptools import find_packages, setup


setup(
    name='django-simple-quiz',
    version='0.1.0',
    author="Mike Dingjan",
    author_email='mike@mikedingjan.nl',
    url='',
    description='Django Quiz App',
    long_description=open('README.md').read(),
    keywords='django, quiz',
    platforms=['osx', 'linux'],
    package_dir={'': 'src'},
    packages=find_packages('src'),
    install_requires=[
        'django-formtools==1.0',
        'django-model-utils==2.5.2',
        'Pillow==3.3.1',
    ],
    include_package_data=True,
    license='Apache License',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
    ],
)



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy


class QuizAppConfig(AppConfig):
    label = 'Quiz'
    name = 'quiz'
    verbose_name = ugettext_lazy("Quiz App")
